echo $CUDA_VISIBLE_DEVICES

## For CPU info:
#
#cat /proc/cpuinfo
#
## For host memory info:
#
#cat /proc/meminfo
#
## For GPU info:
#
#/usr/bin/nvidia-smi -q
#
#module avail nvhpc
#
#module load nvhpc/23.7
#
#pgaccelinfo
#
#nvaccelinfo
